//=======================================================================
//           This file is a part of the Juju GUI application.
//                  Copyright (C) 2011 Nathan Osman
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef CENVIRONMENTSDIALOG_H
#define CENVIRONMENTSDIALOG_H

// Qt includes
#include <QDialog>
#include <QListWidgetItem>
#include <QMap>

// C++ includes
#include <string>

// yaml-cpp includes
#include <yaml-cpp/yaml.h>

// Typedefs to make things a little easier
typedef QMap<QString, QString> StringMap;
typedef QMap<QString, StringMap> EnvironmentMap;

namespace Ui {
    class CEnvironmentsDialog;
}

class CEnvironmentsDialog : public QDialog
{
    Q_OBJECT

public:

    CEnvironmentsDialog(QWidget * = NULL);
    ~CEnvironmentsDialog();

private slots:

    // The buttons
    void OnAdd();
    void OnRemove();
    void OnSetDefault();

    // The listbox
    void OnItemChanged(QListWidgetItem *, QListWidgetItem *);

private:

    // Interface to the config file
    bool LoadConfig();
    bool StoreConfig();
    void AddEnvironmentToQMap(std::string &, const YAML::Node &);

    // GUI utility methods
    void PopulateListbox();
    void ToggleDefault(QString, bool);

    Ui::CEnvironmentsDialog * ui;

    // Filename for the config file
    const char * m_config_file;

    // Contains the config for various environments
    EnvironmentMap m_environments;
    QString m_default_environment;

    // Used to store error messages
    QString m_error;

};

#endif // CENVIRONMENTSDIALOG_H
