//=======================================================================
//           This file is a part of the Juju GUI application.
//                  Copyright (C) 2011 Nathan Osman
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include <CGenericEnvironmentWidget.h>
#include "ui_CGenericEnvironmentWidget.h"

CGenericEnvironmentWidget::CGenericEnvironmentWidget(StringMap & map, QWidget *parent)
    : QWidget(parent), ui(new Ui::CGenericEnvironmentWidget), m_map(map), m_modifying_table(false)
{
    ui->setupUi(this);

    for(StringMap::iterator i = m_map.begin(); i != m_map.end(); ++i)
        AddPairToTable(i.key(), i.value());
}

CGenericEnvironmentWidget::~CGenericEnvironmentWidget()
{
    delete ui;
}

void CGenericEnvironmentWidget::OnAdd()
{
    // Add the item to the table
    AddPairToTable("key", "value");

    // Add the item to the map
    m_map.insert("key", "value");
}

void CGenericEnvironmentWidget::OnRemove()
{
    // Get the currently selected row
    int current_row = ui->Table->currentRow();

    if(current_row >= 0)
    {
        // Remove the item from the map
        QString key = ui->Table->item(current_row, 0)->text();
        m_map.remove(key);

        // Remove the item from the table
        ui->Table->removeRow(current_row);
    }
}

void CGenericEnvironmentWidget::OnItemChanged(int row, int column)
{
    if(!m_modifying_table)
    {
        QString key = ui->Table->item(row, 0)->text();

        // Check to see if the column is being changed
        if(column == 0)
        {
            QString old_key = ui->Table->item(row, 0)->data(Qt::UserRole).toString();
            m_map.remove(old_key);
            m_map.insert(key, ui->Table->item(row, 1)->text());
        }
        else
            m_map[key] = ui->Table->item(row, 1)->text();
    }
}

/// Adds a key / value pair to the table.
/**
  * \param key the key
  * \param value the value for the provided key
  */
void CGenericEnvironmentWidget::AddPairToTable(QString key, QString value)
{
    m_modifying_table = true;

    // Insert a new row
    ui->Table->insertRow(ui->Table->rowCount());

    // Create the items
    QTableWidgetItem * key_item   = new QTableWidgetItem(key);
    QTableWidgetItem * value_item = new QTableWidgetItem(value);
    key_item->setData(Qt::UserRole, key);

    ui->Table->setItem(ui->Table->rowCount() - 1, 0, key_item);
    ui->Table->setItem(ui->Table->rowCount() - 1, 1, value_item);

    m_modifying_table = false;
}
